module Character exposing (..)

import Sphere exposing (Sphere)
import Json.Decode exposing (int)
import Sphere exposing (getDotsOfSphereInList)
import BoundedInt
import Html exposing (s)

type alias Character =
    { name : String
    , creationPoints : CreationPoints
    , spheres : List Sphere
    }

type alias CreationPoints =
    { freebie : Int
    , spheres : Int
    }

changeName : Character -> String -> Character
changeName character newName =
    if ( newName == "")
    then
        character
    else
        { character | name = newName }

changeSphere : Character -> Sphere -> Character
changeSphere character sphere =
    let
        maybeOldDots = getDotsOfSphereInList character.spheres sphere.name
        difference = case maybeOldDots of
            Nothing -> 0
            Just oldDots ->
                sphere.dots.value - oldDots.value
    in
        case maybeOldDots of
            Nothing -> character
            Just oldDots ->
                if difference >= 0
                then
                    let
                        sA = 
                            if character.creationPoints.spheres > 0
                                then min difference character.creationPoints.spheres
                                else 0
                        fA = min ( difference - sA ) ( character.creationPoints.freebie // Sphere.freebieCost )
                    in
                        { character
                        | spheres = Sphere.changeSphereInList character.spheres 
                            { sphere 
                            | dots = BoundedInt.changeBy oldDots ( fA + sA )
                            } 
                        , creationPoints =
                            { freebie = character.creationPoints.freebie - fA * Sphere.freebieCost
                            , spheres = character.creationPoints.spheres - sA - fA
                            }
                        }
                else
                    let
                        fA =
                            if character.creationPoints.spheres < 0
                            then negate ( max difference character.creationPoints.spheres )
                            else 0
                    in
                        { character
                        | spheres = Sphere.changeSphereInList character.spheres sphere
                        , creationPoints =
                            { freebie = character.creationPoints.freebie + fA * Sphere.freebieCost
                            , spheres = character.creationPoints.spheres - difference
                            }
                        }
        -- this currently works incorrectly

new : Character
new =
    { name = "Default Name" 
    , creationPoints =
        { freebie = 15
        , spheres = 6
        }
    , spheres = 
        [   { name = "Correspondence"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ,   { name = "Entropy"
            , dots = Sphere.dots 0
            , affinity = False
            }
        ]
    }